package com.client.web.helper;

import java.beans.Statement;
import java.lang.reflect.Method;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class WSServlet extends HttpServlet
{
	final String tomcatLoader = "org.apache.catalina.loader.WebappClassLoader";
	final String jBossLoader = "org.jboss.web.tomcat.service.WebAppClassLoader";
	final String jBoss510Loader = "org.jboss.web.tomcat.service.WebCtxLoader$ENCLoader";
	
	private Class cls = null;
	private Method doGetMethod = null;
	private Method doPostMethod = null;
	private Method doPutMethod = null;
	private Method doDeleteMethod = null;
	private Method initMethod = null;

	private HttpServlet wsServlet = null;
	
	private static Log log = LogFactory.getLog( "com.adminserver.web.helper.WSServlet" );
	
	private String clsLoaderName;
	
	public WSServlet()
	{
		clsLoaderName = Thread.currentThread().getContextClassLoader().getClass().getName();
		if (clsLoaderName.equalsIgnoreCase(tomcatLoader) || clsLoaderName.equalsIgnoreCase(jBossLoader) || clsLoaderName.equalsIgnoreCase(jBoss510Loader))
		{
			try
			{
				cls = Class.forName("com.sun.xml.ws.transport.http.servlet.WSServlet");
				wsServlet = (HttpServlet)cls.newInstance();
				Class[] parameterTypes = {HttpServletRequest.class, HttpServletResponse.class};
				doGetMethod = cls.getDeclaredMethod("doGet", parameterTypes);
				doPostMethod = cls.getDeclaredMethod("doPost", parameterTypes);
				doPutMethod = cls.getDeclaredMethod("doPut", parameterTypes);
				doDeleteMethod = cls.getDeclaredMethod("doDelete", parameterTypes);
				initMethod = cls.getDeclaredMethod("init", new Class[]{ServletConfig.class});
			}
			catch(ClassNotFoundException e)
			{
				//throw e;
				log.error("ClassNotFound exception while creating WS servlet ..");
				e.printStackTrace();
			}
			catch(Exception e)
			{
				log.error("Exception while creating WS servlet ..");
				e.printStackTrace();
			}
		}
	}
	
    public void init(ServletConfig servletConfig) throws ServletException {
        super.init(servletConfig);
		if (clsLoaderName.equalsIgnoreCase(tomcatLoader) || clsLoaderName.equalsIgnoreCase(jBossLoader) || clsLoaderName.equalsIgnoreCase(jBoss510Loader))
		{
			initMethod.setAccessible(true);
			try{
				initMethod.invoke(wsServlet, new Object[]{servletConfig} );
			}
			catch(Exception e)
			{
				log.error("Exception in init method");
				e.printStackTrace();
			}
		}
    }
	
	protected void doPost( HttpServletRequest request, HttpServletResponse response) throws ServletException {
		try{
			if(doPostMethod != null)
			{
				doPostMethod.setAccessible(true);
				doPostMethod.invoke(wsServlet, new Object[]{request, response} );
			}
		}
		catch(Exception e)
		{
			log.error("Exception in doPost method");
			e.printStackTrace();
		}
	}
	
	protected void doGet( HttpServletRequest request, HttpServletResponse response) throws ServletException {
		try{
			if(doGetMethod != null)
			{
				doGetMethod.setAccessible(true);
				doGetMethod.invoke(wsServlet, new Object[]{request, response} );
			}
		}
		catch(Exception e)
		{
			log.error("Exception while executing doGet method on servlet ..");
			e.printStackTrace();
		}
	}

	protected void doPut( HttpServletRequest request, HttpServletResponse response) throws ServletException {
		try{
			if(doPutMethod != null)
				doPutMethod.setAccessible(true);
				doPutMethod.invoke(wsServlet, new Object[]{request, response} );
		}
		catch(Exception e)
		{
			log.error("Exception while executing doPut method on servlet ..");
			e.printStackTrace();
		}
		
	}
	
	protected void doDelete( HttpServletRequest request, HttpServletResponse response) throws ServletException {
		try{
			if(doDeleteMethod != null)
				doDeleteMethod.setAccessible(true);
				doDeleteMethod.invoke(wsServlet, new Object[]{request, response} );
		}
		catch(Exception e)
		{
			log.error("Exception while executing doDelete method on servlet ..");
			e.printStackTrace();

		}
	}
}
